/**
 *
 * @source: http://bitbucket.org/openattic/openattic
 *
 * @licstart  The following is the entire license notice for the
 *  JavaScript code in this page.
 *
 * Copyright (c) 2017 SUSE LLC
 *
 *
 * The JavaScript code in this page is free software: you can
 * redistribute it and/or modify it under the terms of the GNU
 * General Public License as published by the Free Software
 * Foundation; version 2.
 *
 * This package is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * As additional permission under GNU GPL version 2 section 3, you
 * may distribute non-source (e.g., minimized or compacted) forms of
 * that code without the copy of the GNU GPL normally required by
 * section 1, provided you include this license notice and a URL
 * through which recipients can access the Corresponding Source.
 *
 * @licend  The above is the entire license notice
 * for the JavaScript code in this page.
 *
 */
"use strict";

/**
 * Defines task queue elements and common task queue related functions.
 *
 * Defines the following elements:
 * - taskQueue - element to click to open the dialogue.
 * - dialog.tabs - Holds the name, elements, column descriptions of each tab.
 * - dialog.modalElements - Holds the default elements that exist in all tabs.
 * - deletionDialog - Holds the different texts and needed elements.
 *
 * Defines the following common functions:
 * - changeTab(String tab) - Changes to the given tab.
 * - open() - Opens the task queue dialog.
 * - close() - Closes the task queue dialog.
 * - waitForPendingTasks() - Waits for all pending tasks to finish, use it with care because it's recursive.
 * - createTask(int time) - Triggers an API-Call to create a test task with a specified time.
 * - deleteTasks(String tab [, String name]) - Deletes all or the given task name in the given tab.
 */
(function () {
  const self = {};
  const helpers = require("../../common.js");

  // The element to click to open the modal dialogue.
  self.taskQueue = element(by.className("tc_task-queue"));
  self.loadingElem = element(by.className("tc_loading_pending"));

  // Describes the dialog elements.
  self.dialog = {
    tabs: {
      pending: { // self.is the same attribute key used by the UI.
        name: "Pending",
        elements: {}, // Will be filled with protractor elements for each tab.
        columns: {
          description: { // self.is the same attribute key that's used by the API-Object.
            name: "Name"
            // 'element: protractorElement' will be inserted into each column.
          },
          created: {
            name: "Created"
          },
          percent: {
            name: "Complete"
          },
          approx: {
            name: "Estimated"
          }
        }
      },
      failed: {
        name: "Failed",
        elements: {},
        columns: {
          description: {
            name: "Name"
          },
          status: {
            name: "Reason"
          },
          created: {
            name: "Created"
          },
          approx: {
            name: "Runtime"
          },
          last_modified: {
            name: "Failed"
          }
        }
      },
      finished: {
        name: "Finished",
        elements: {},
        columns: {
          description: {
            name: "Name"
          },
          created: {
            name: "Created"
          },
          approx: {
            name: "Runtime"
          },
          last_modified: {
            name: "Finished"
          }
        }
      }
    },
    modalElements: { // Holds protractorElements.
      header: element(by.className("openattic-modal-header")),
      content: element(by.className("openattic-modal-content")),
      footer: element(by.className("openattic-modal-footer")),
      closeBtn: element(by.className("modal-close-btn"))
    },
    tabElements: { // self.is used to create elements for each tab for structure creation. (Internal use)
      tab: "tc_tab_",
      deleteBtn: "tc_task_delete_",
      loadingParagraph: "tc_loading_",
      refreshBtn: "tc_refreshBtn_",
      noElements: "tc_no_elements_",
      listing: "tc_listing_",
      selectAll: "tc_select_all_"
    }
  };

  self.movedElements = {
    close: element(by.className("tc-tab-del-close")),
    movedTasks: element.all(by.css("uib-accordion.tc-moved-tasks ul > li"))
  };

  /**
   * Adds elements to each tab.elements and adds an element to each column of the tab.
   * For internal use only.
   */
  Object.keys(self.dialog.tabs).forEach(tabName => {
    const tabElements = self.dialog.tabElements;
    const tab = self.dialog.tabs[tabName];
    /**
     * Uses dialog.tabElements to combine it with the tab names to create the tab elements.
     */
    Object.keys(tabElements).forEach(elementName => {
      tab.elements[elementName] = element(by.className(tabElements[elementName] + tabName));
    });

    /**
     * Uses the attribute name of each column and tab name to create an column element.
     */
    Object.keys(tab.columns).forEach(columnName => {
      tab.columns[columnName].element = element(by.className("tc-col-" + tabName + "-" + columnName));
    });
  });

  /**
   * Holds the different texts and needed elements of the deletion dialog.
   */
  self.deletionDialog = {
    text: {
      warning: "If you delete running tasks, it will abort the execution and won't roll back what has been done " +
        "so far!",
      multiDelete: "You are about to delete multiple tasks."
    },
    element: {
      warning: element(by.className("tc-run-warn")),
      singleDelete: element(by.className("tc_delete_one")),
      multiDelete: element(by.className("tc_delete_multiple")),
      confirmBtn: element(by.className("tc-tab-del-confirm"))
    }
  };

  /*
   * Expects default buttons to be displayed or not.
   * @param {Boolean} displayed - What to expect.
   */
  self.expectDefaultModalElements = displayed => {
    const elements = self.dialog.modalElements;
    Object.keys(elements).forEach(elementName => {
      const modalElement = elements[elementName];
      expect(modalElement.isPresent()).toBe(displayed);
      if (displayed) {
        expect(modalElement.isDisplayed()).toBe(displayed);
      }
    });
  };

  /**
   * Changes to the given tab and expect default elements to be there.
   * @param {String} tabName
   */
  self.changeTab = tabName => {
    // browser.sleep(helpers.configs.sleep / 2);
    let elem = self.dialog.tabs[tabName].elements.tab;
    helpers.waitForElementVisible(elem);
    elem.click();
    self.expectDefaultModalElements(true);
    helpers.waitForElementInvisible(self.dialog.tabs[tabName].elements.loadingParagraph);
    // browser.sleep(helpers.configs.sleep / 2);
  };

  /**
   * Opens the task queue dialog and expect default elements to be there.
   */
  self.open = function () {
    self.taskQueue.click();
    self.expectDefaultModalElements(true);
  };

  /**
   * Closes the task queue dialog.
   */
  self.close = () => {
    browser.sleep(helpers.configs.sleep / 2);
    element(by.className("modal-close-btn")).click();
    self.expectDefaultModalElements(false);
  };

  /*
   * Will delete all tasks.
   * Expecting the dialog to be closed.
   */
  self.deleteAllTasks = () => {
    self.open();
    Object.keys(self.dialog.tabs).forEach(function (tabName) { // => [pending, failed, finished]
      self.changeTab(tabName);
      self.dialog.tabs[tabName].elements.listing.isDisplayed().then(function (displayed) {
        if (displayed) { // If at least one task is there the listing is shown.
          self.deleteTasks(tabName);
        }
      });
    });
    self.close();
  };

  /**
   * Validates which tab is opened by the task queue.
   * Expecting the dialog to be closed.
   * @param {string} tabName - Which tab to be expect.
   */
  self.validateDisplayedTab = tabName => {
    self.open();
    helpers.waitForElementVisible(self.dialog.tabs[tabName].elements.deleteBtn);
    expect(self.dialog.tabs[tabName].elements.deleteBtn.isDisplayed()).toBe(true);
    self.close();
  };

  /**
   * Validates the text shown by the given tab when you open the task queue.
   * Expecting the dialog to be closed.
   * @param {string} tabName - Which tab text to get.
   * @param {string} tabText - Which text will be expected in the label.
   */
  self.validateTabName = (tabName, tabText) => {
    self.open();
    let elem = self.dialog.tabs[tabName].elements.tab;
    helpers.waitForElement(elem.element(by.linkText(tabText)));
    expect(elem.getText()).toBe(tabText);
    self.close();
  };

  /**
   * Waits for all pending tasks to finish.
   * Expects the dialog to be closed.
   */
  self.waitForPendingTasks = () => {
    self.open(); // Opens the dialog
    self.changeTab("pending");
    helpers.waitForElementVisible(self.dialog.tabs.pending.elements.noElements); //Make sure the element is visible
    self.close(); // Closes the dialog when there are zero pending tasks.
  };

  /**
   * Triggers an API-Call to create a test task with a specified time.
   * It executes an XMLHttpRequest to the api with the login data provided by configs.js.
   * @param {int} times - One round takes around 1 1/2 seconds.
   * @param {int} [quantity] - How many task should be created..
   */
  self.createTask = (times, quantity) => {
    quantity = quantity || 1;
    for (let x = 0; x < quantity; x++) {
      browser.executeScript(function (innerTimes, configs) {
        const xhr = new XMLHttpRequest();
        const parts = configs.urls.base.split("/");
        const url = parts[0] + "//" + configs.username + ":" + configs.password + "@" +
          parts[2] + configs.urls.api + "taskqueue/test_task";
        xhr.open("post", url, true);
        xhr.setRequestHeader("Content-Type", "application/json");
        xhr.send(JSON.stringify({times: innerTimes}));
      }, times, helpers.configs);
    }
  };

  /**
   * Deletes all or the given task name in the given tab.
   * @param {String} tabName - Name of the tasks where the deletion takes place and which is displayed.
   * @param {String} [taskName] - Name of the task to delete.
   */
  self.deleteTasks = (tabName, taskName) => {
    self.changeTab(tabName);
    if (taskName) { // If a singel deletion takes place.
      helpers.waitForElementVisible(element.all(by.cssContainingText("tr", taskName)).first());
      const task = element.all(by.cssContainingText("tr", taskName)).first();
      expect(task.isDisplayed()).toBe(true);
      task.click();
    } else { // Delete all tasks in the tabName.
      helpers.waitForElementVisible(self.dialog.tabs[tabName].elements.selectAll);
      self.dialog.tabs[tabName].elements.selectAll.click();
    }
    let itemLength = 1;
    self.dialog.tabs.pending.elements.tab.getText().then(function (s) {
      itemLength = parseInt(s.match(/[0-9]+/)[0], 10);
    });
    self.dialog.tabs[tabName].elements.deleteBtn.click();
    self.handleDeleteForm(tabName, itemLength);
  };

  /**
   * Is used to determine if everything is alright while deleting.
   * You should't call self.function from outside.
   * @param {String} tabName - Name of the current tab.
   * @param {String} itemLength - Count of how many items being deleted.
   */
  self.handleDeleteForm = (tabName, itemLength) => {
    const dialog = self.deletionDialog;
    const elements = dialog.element;
    const warning = elements.warning;
    const singleDelete = elements.singleDelete;
    const confirmBtn = elements.confirmBtn;
    const showWarning = tabName === "pending";
    // Should only show the warning if you want to delete a pending task.
    expect(warning.isDisplayed()).toBe(showWarning);
    if (showWarning) {
      expect(warning.getText()).toBe(dialog.text.warning);
    }
    elements.multiDelete.isDisplayed().then(function (displayed) {
      // If you delete multiple tasks you the single deletion can't be shown and vice versa.
      expect(singleDelete.isDisplayed()).toBe(!displayed);
    });
    if (itemLength > 1) {
      expect($$("uib-accordion.tc-tasks-to-delete ul > li").count()).toBe(itemLength);
    }
    expect(confirmBtn.isDisplayed()).toBe(true);
    expect(confirmBtn.getText()).toBe("Delete");
    confirmBtn.click();
    browser.sleep(helpers.configs.sleep / 2);
  };
  module.exports = self;
}());

